/* 
 * This file must be loaded after utils.
 */

/**
 * Set page title and description
 * @param {type} title
 * @param {type} description
 * @returns {undefined}
 */
function setPageTitleDescription(title, description, noHeader){
    document.title = document.title + " " + jQuery('<p>' + title + '</p>').text();
    if(noHeader === true){
        $(".page-title").hide();
    }
    $("#page-header-title").html(title);
    $("#page-header-description").html(description);
}

/**
 * Set's page info.
 * @returns {undefined}
 */
function preparePageInfo(){
    if ($("#pageinfotext").length>0) {
        $("#pageinfoicon").show();
        if(!$("#pageinfotext").hasClass("created")){
            $("#pageinfocontent").empty();
            $("#dialog-pageinfo-title").text($("#page-header-title").text());
            $("#pageinfocontent").html($("#pageinfotext").html());
            $("#pageinfotext").remove();
        }
        $("#pageinfoicon").on("click",function(){
            if($("#dialog-pageinfo").is(':visible')){
                $("#dialog-pageinfo").modal("hide");
            } else {
                $("#dialog-pageinfo").modal("show");
            }
        });
    } else {
        $("#pageinfoicon").hide();
        $("#pageinfoicon").off("click");
    }
}

/**
 * Yes no confirmation box.
 * @param {type} title
 * @param {type} message
 * @param {type} callback
 * @param {type} yesText
 * @param {type} noText
 * @param {type} noCallBack
 * @returns {undefined}
 */
function yesnoConfirmation(title, message, callback, yesText, noText, noCallBack){
    $("body").append('\n\
        <div class="modal fade" id="dialog-yesnoconfirmation" tabindex="-1" role="dialog" aria-labelledby="yesnoconfirmation" aria-hidden="true">\n\
            <div class="modal-dialog">\n\
                <div class="modal-content">\n\
                    <div class="modal-header">\n\
                        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>\n\
                        <h4 class="modal-title" id="dialog-yesnoconfirmation-title">'+title+'</h4>\n\
                    </div>\n\
                    <div class="modal-body" id="pageinfocontent">\n\
                    '
                     + message +
                    '\n\
                    </div>\n\
                    <div class="modal-footer clearfix">\n\
                        <button type="button" class="btn btn-success" id="dialog-yesnoconfirmation-confirmed">'+((typeof yesText==="undefined")?"Yes":yesText)+'</button>\n\
                        <button type="button" class="btn btn-danger" id="dialog-yesnoconfirmation-dismissed" data-dismiss="modal">'+((typeof noText==="undefined")?"No":noText)+'</button>\n\
                    </div>\n\
                </div>\n\
            </div>\n\
        </div>');
    $("#dialog-yesnoconfirmation form").submit(function(event){
        event.preventDefault();
        var result = callback();
        if(result !== false){
            $("#dialog-yesnoconfirmation").modal("hide");
        }
    });

    $('#dialog-yesnoconfirmation').on('hidden.bs.modal', function (e) {
        $(this).remove();
    })
    $("#dialog-yesnoconfirmation-confirmed").on("click", function(){
        var result = callback();
        if(result !== false){
            $("#dialog-yesnoconfirmation").modal("hide");
        }
    });
    $("#dialog-yesnoconfirmation-dismissed").on("click", function(){
        if(typeof noCallBack !== "undefined" && noCallBack !== null){
            var result = noCallBack();
            if(result !== false){
                $("#dialog-yesnoconfirmation").modal("hide");
            }
        } else {
            $("#dialog-yesnoconfirmation").modal("hide");
        }
    });
    $("#dialog-yesnoconfirmation").modal("show");
}
/**
 * Simple confirmation dialog.
 * @param {type} title
 * @param {type} message
 * @param {type} callback
 * @param {type} closeText
 * @returns {undefined}
 */
function simpleDialog(title, message, callback, closeText){
    $("body").append(
        '<div class="modal fade" id="dialog-simpledialog" tabindex="-1" role="dialog" aria-labelledby="simpledialog" aria-hidden="true">'+
            '<div class="modal-dialog">'+
                '<div class="modal-content">'+
                    '<div class="modal-header">'+
                        '<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>'+
                        '<h4 class="modal-title" id="dialog-simpledialog-title">'+title+'</h4>'+
                    '</div>'+
                    '<div class="modal-body" id="simpledialog-content">'+
                    message +
                    '</div>'+
                    '<div class="modal-footer">'+
                        '<button type="button" class="btn btn-success" id="dialog-simpledialog-confirmed">'+((typeof closeText==="undefined")?"Close":closeText)+'</button>'+
                    '</div>'+
                '</div>'+
            '</div>'+
        '</div>');
    $("#dialog-simpledialog form").submit(function(event){
        event.preventDefault();
        var result = callback();
        if(result !== false){
            $("#dialog-simpledialog").modal("hide");
        }
    });
    $('#dialog-simpledialog').on('hidden.bs.modal', function (e) {
        $(this).remove();
    })
    $("#dialog-simpledialog-confirmed").on("click", function(){
        var result = callback();
        if(result !== false){
            $("#dialog-simpledialog").modal("hide");
        }
    });
    $("#dialog-simpledialog").modal("show");
}

/////////////////////////////////////////////
//// functions for visualization by drivers and plugins.

function WebPresentation() {
    this.functionDataCollection = new Array();

}

WebPresentation.prototype.createDataSet = function (headerName, getUrl, devicesFetchUrl, contentDiv, callBackCustomFunction, callBackDeviceFunction) {
    var self = this;
    $.get(getUrl).success(function (swData) {
        var html = '';
        try {
            var data = swData.result.data;
            for (var i = 0; i < data.presentation.length; i++) {
                var presentData = data.presentation[i];
                html += '<div class="row">\n\
                            <div class="col-xs-12">\n\
                                <div class="x_panel">\n\
                                    <div class="x_title">\n\
                                        <h2>' + presentData.title + '</h2>\n\
                                        <div class="clearfix"></div>\n\
                                    </div>\n\
                                    <div class="x_content">\n\
                         <div class="row"><div class="col-xs-12"><p>' + presentData.description + '</p></div></div>';
                for (var j = 0; j < presentData.content.length; j++) {
                    var content = presentData.content[j];
                    var endData = "";
                    switch (content.type) {
                        case "SIMPLE_NVP":
                            if (content.label === "custom_driver_function") {
                                var UUID = createUUID().replace(/-/g, '');
                                endData = self.createCustomDriverFunction(UUID, content.content);
                            } else {
                                html += '<div class="row">\n\
                                            <div class="col-xs-3">' + content.label + '</div>\n\
                                            <div class="col-xs-9">' + content.content + '</div>\n\
                                        </div>';
                            }
                            html += endData;
                            break;
                        case "LIST_NVP":
                            for (var k = 0; k < content.content.length; k++) {
                                for (var label in content.content[k]) {
                                    if (label === "custom_driver_function") {
                                        try {
                                            var UUID = createUUID().replace(/-/g, '');
                                            endData = self.createCustomDriverFunction(UUID, content.content[k][label]);
                                        } catch (err) {
                                            alert(err);
                                        }
                                    } else {
                                        html += '<div class="row">\n\
                                                    <div class="col-xs-3">' + label + '</div>\n\
                                                    <div class="col-xs-9">' + content.content[k][label] + '</div>\n\
                                                </div>';
                                    }
                                }
                            }
                            html += endData;
                            break;
                        case "COMPLEX_NVP":
                            for (var key in content.content) {
                                var toggleId = createUUID();
                                var contentSet = content.content[key];
                                html += '<div class="panel-group">\n\
                                            <div class="panel panel-primary">\n\
                                                <div class="panel-heading">\n\
                                                    <h4 class="panel-title" id="collapse'+ toggleId +'-control" style="cursor:pointer;">'
                                                        + key + 
                                                    '</h4>\n\
                                                </div>\n\
                                                <div id="collapse'+ toggleId +'-collapse" class="panel-collapse collapse panel-body">';
                                for (var k = 0; k < contentSet.length; k++) {
                                    for (var n in contentSet[k]) {
                                        if (n === "custom_driver_function") {
                                            var UUID = createUUID().replace(/-/g, '');
                                            endData = self.createCustomDriverFunction(UUID, contentSet[k][n]);
                                        } else {
                                            html += '<div class="row">\n\
                                                        <div class="col-xs-3">' + n + '</div>\n\
                                                        <div class="col-xs-9">' + contentSet[k][n] + '</div>\n\
                                                    </div>';
                                        }
                                    }
                                    html += '<hr size="2px"/>';
                                }
                                html += endData;
                                html += '</div></div>\n\
</div><script>$("#collapse'+ toggleId +'-control").off("click").on("click", function(){ $("#collapse'+ toggleId +'-collapse").collapse("toggle"); });<\/script>';
                            }
                            break;
                    }
                }
                html += '</div></div></div></div></div>';
            }
        } catch (err) {
            html += '<div class="row"><div class="col-xs-12"><p>No information available</p></div></div>';
        }
        html += '';
        $('#' + contentDiv).html(html);
        $(".customFunctionExec").off('click').on('click', function () {
            var selectedFuncId = $(this).attr("id");
            for (var i = 0; i < self.functionDataCollection.length; i++) {
                if (self.functionDataCollection[i].id === selectedFuncId) {
                    var funcData = self.functionDataCollection[i].content;
                    try {
                        callBackCustomFunction(funcData);
                    } catch (err) {
                        showErrorMessage("Function", "Function not executed " + err);
                    }
                }
            }
        });
        $(".mutationSelectDevice").off('click').on('click', function () {
            var selectedFuncId = $(this).attr("id");
            if ($('#selectDeviceItemWindow').length === 0) {
                $("#" + contentDiv).append('<div class="modal fade" id="selectDeviceItemWindow" tabindex="-1" role="dialog" aria-hidden="true">'+
                                           '<div class="modal-dialog">'+
                                               '<div class="modal-content">'+
                                                   '<div class="modal-header">'+
                                                       '<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>'+
                                                       '<h4 class="modal-title">Select the device to use</h4>'+
                                                   '</div>'+
                                                   '<div class="modal-body">'+
                                                       '<div class="row">'+
                                                           '<div class="col-xs-12">'+
                                                               '<p>Select the device to use. If the specified device does not exist please add it or create one with the visual device editor first.</p>'+
                                                           '</div>'+
                                                       '</div>'+
                                                       '<div class="form form-horizontal">'+
                                                           '<div class="form-group">'+
                                                               '<label for="useDeviceSelection" class="control-label">Device name</label>'+
                                                               '<select id="useDeviceSelection" class="form-control"></select>'+
                                                           '</div>'+
                                                           '<div class="form-group">'+
                                                               '<label for="useDeviceSelectionName" class="control-label">Device name</label>'+
                                                               '<input type="text" data-type="alphanumeric" name="useDeviceSelectionName" class="form-control" id="useDeviceSelectionName" value="" placeholder="Enter a name for this device" required />'+
                                                           '</div>'+
                                                           '<div class="form-group">'+
                                                               '<label for="useDeviceSelectionLocation" class="control-label">Location</label>'+
                                                               '<select id="useDeviceSelectionLocation" class="form-control"></select>'+
                                                           '</div>'+
                                                           '<div class="form-group">'+
                                                               '<label for="useDeviceSelectionCategory" class="control-label">Category</label>'+
                                                               '<select id="useDeviceSelectionCategory" class="form-control"></select>'+
                                                           '</div>'+
                                                       '</div>'+
                                                   '</div>'+
                                                   '<div class="modal-footer">'+
                                                       '<button name="selectDeviceButton" id="selectDeviceButton" class="btn btn-success" data-dismiss="modal">Select and start</button>' +
                                                       '<button name="cancelSelectDeviceButton" id="cancelSelectDeviceButton" class="btn btn-danger" data-dismiss="modal">Cancel</button>' +
                                                   '</div>'+
                                               '</div>'+
                                           '</div>'+
                                       '</div>');

                getHttpJsonRPC(allDevicesList,function(resultData) {
                    for(var q=0;q<resultData.length;q++){
                        $("#useDeviceSelection").append('<option value="'+resultData[q].id+'">'+resultData[q].name+'</option>');
                    }
                });
                getHttpJsonRPC('{"jsonrpc": "2.0", "method": "LocationService.getLocations", "id":"LocationService.getLocations"}',function(resultData) {
                    for(var q=0;q<resultData.length;q++){
                        $("#useDeviceSelectionLocation").append('<option value="'+resultData[q].id+'">'+resultData[q].name+'</option>');
                    }
                });
                getHttpJsonRPC('{"jsonrpc":"2.0", "id": "CategoryService.getFullCategoryList","method": "CategoryService.getFullCategoryList"}',function(resultData) {
                    for(var q=0;q<resultData.length;q++){
                        $("#useDeviceSelectionCategory").append('<option value="'+resultData[q].id+'">'+resultData[q].name+'</option>');
                    }
                });

                $("#selectDeviceButton").off('click').on('click', function (e) {
                    e.preventDefault();
                    for (var i = 0; i < self.functionDataCollection.length; i++) {
                        if (self.functionDataCollection[i].id === selectedFuncId) {
                            if ($("#useDeviceSelection").val() === "") {
                                showErrorMessage("Selection error", "Select a correct device or cancel to dismiss");
                            } else {
                                var funcData = self.functionDataCollection[i].content;
                                funcData["device_name"] = $("#useDeviceSelectionName").val();
                                funcData["device_locationid"] = parseInt($("#useDeviceSelectionLocation").val());
                                funcData["device_categoryid"] = parseInt($("#useDeviceSelectionCategory").val());
                                try {
                                    funcData["device_id"] = parseInt($("#useDeviceSelection").val());
                                    callBackDeviceFunction(funcData);
                                } catch (err) {
                                    showErrorMessage("Device select error", "Incorrect device information, contact author.");
                                }
                                $("#selectDeviceItemWindow").modal("hide");
                            }
                        }
                    }
                });
            }
            $("#selectDeviceItemWindow").modal("show");
        });
    }, "json").error(function (jqXHR, textStatus, errorThrown) {
        var html = '<div class="driversnvp" style="width:100%;"><h2>' + headerName + '</h2>';
        html += "<p>Could not interpret data received from server</p>";
        html += "<p>Error: " + textStatus + ", " + errorThrown + "</p>";
        $('#' + contentDiv).html(html);
    });
};


WebPresentation.prototype.createCustomDriverFunction = function (UUID, functionData) {
    var data = {id: "func_" + UUID, type: functionData.function_id, content: functionData};
    this.functionDataCollection.push(data);
    var html = '<div class="nvp">';
    switch (functionData.function_id) {
        case "addExistingNewDevice":
            html += '<button style="float:right; margin:5px; " class="btn btn-info mutationSelectDevice" id="func_' + UUID + '" name="mutationSelectDevice">Select from device list</button>';
            break;
        case "addAndCreateNewDevice":
            html += '<button style="float:right; margin:5px; " class="btn btn-info mutationAddNewDevice" id="func_' + UUID + '" name="mutationAddNewDevice">Auto create device</button>';
            break;
        case "customFunction":
            html += '<button style="float:right; margin:5px; " class="btn btn-info customFunctionExec" id="func_' + UUID + '" name="customFunctionExec">' + functionData.function_label + '</button>';
            break;
    }
    html += '</div>';
    return html;
}

/**
 * This code is used for creating routines for selecting items.
 * On the page there will be a function needed which is called as callback when a
 * selection is made and will be passed as a javascript object. The page creator has to 
 * take the different objects into account. This functions makes it possible to use the 
 * exact same modal on different pages. The base of the selection is made from the trigger RPC
 * where the trigger RPC takes lead of possible items to be added and because the triggering
 * system arranges all possible items in the server it is possible to apply the trigger item
 * sources to multiple different in server engines like the macro's because they use the same
 * execution styles.
 * Filters can be applied for example for devices to limit the command field types.
 * @returns {itemSelectionModal}
 */

function ItemSelectionModal(){
    
    this.callBack = null;
    this.options = { "devicefilter" :"[null]", "devicetypefilter" : null};
    this.selectionType = "match";
    this.callBackData = {"itemType" : "null"};
    
    this.modalComposed = false;
    
    var self = this;
    
    if ($('#addItemWindow').length === 0) {
        $("body").append('<div class="modal" id="addItemWindow" style="z-index:10000;">'+
        '<div class="modal-dialog">'+
          '<div class="modal-content">'+
            '<div class="modal-header">'+
              '<button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>'+
              '<h4 class="modal-title">Item selection</h4>'+
            '</div>'+
            '<div class="modal-body">'+
                '<div class="row">' +
                    '<div class="col-md-4">' +
                        '<div class="panel panel-default">'+
                            '<div class="panel-heading">'+
                                '<h3 class="panel-title">Type</h3>'+
                            '</div>'+
                            '<div class="panel-body">Choose</div>'+
                                '<div class="list-group" id="ItemSelectionModalTypes">' +
                                '</div>' +
                        '</div>'+
                    '</div>' +
                    '<div class="col-md-8">' +
                        '<div class="panel panel-default">'+
                            '<div class="panel-heading">'+
                                '<h3 class="panel-title" id="ItemSelectionModalTypeName">Select type</h3>'+
                            '</div>'+
                            '<div class="panel-body">'+
                                '<div id="itemSelectionMatchContent"></div>' +
                            '</div>' +
                        '</div>'+
                    '</div>' +
                '</div>' +
            '</div>'+
            '<div class="modal-footer">'+
              '<button type="button" class="btn btn-info" style="display:none;" id="addMatchItem">Add item</button>'+
              '<button type="button" class="btn btn-danger" data-dismiss="modal" id="cancelAddMatchItem">Cancel</button>'+
            '</div>'+
          '</div>'+
        '</div>'+
      '</div>');
    }
    
    this.sourceDiv = $('#addItemWindow');
    
    $('#addMatchItem').off('click').on('click', function() {
        self.sourceDiv.modal('hide');
        if(typeof self.callBack!=="undefined" && self.callBack!==null) self.callBack(self.callBackData);
        self.callBackData = {"itemType" : "null"};
    });
    
}
/**
 * Opens the modal
 * @returns {undefined}
 */
ItemSelectionModal.prototype.open = function() {
    if(this.modalComposed===false){
        this.createModal();
        this.modalComposed = true;
    }
    $('#addMatchItem').hide();
    $("#ItemSelectionModalTypeName").html('<span class="glyphicon glyphicon-chevron-left"></span>Select type');
    $("#itemSelectionMatchContent").empty();
    this.sourceDiv.modal('show');
};

/**
 * Sets options data.
 * For example sets the type of device fields to return.
 * @param {type} optionData
 * @returns {undefined}
 */
ItemSelectionModal.prototype.setOptions = function (optionData){
    try { this.options.devicefilter = optionData.deviceFilter; } catch(err){}
    try { if(optionData.deviceTypeFilter!==undefined) this.options.devicetypefilter = optionData.deviceTypeFilter; } catch(err){}
};

/**
 * Sets the callback function to be called when a selection is made.
 * @param {type} callBackFunction
 * @returns {undefined}
 */
ItemSelectionModal.prototype.setCallBack = function (callBackFunction){
    this.callBack = callBackFunction;
};

/**
 * Sets the callback function to be called when a selection has been made.
 * @param {type} data
 * @returns {undefined}
 */
ItemSelectionModal.prototype.runCallBack = function (data){
    this.callBack(data);
};

/**
 * Sets the selection type.
 * Set type to match to retrieve the data fields of a device, set to action to retrieve command fields.
 * @param {type} type
 * @returns {undefined}
 */
ItemSelectionModal.prototype.setSelectionType = function (type){
    this.selectionType = type;
};

/**
 * Creates the initial modal and calls all functions on selection
 * @returns {undefined}
 */
ItemSelectionModal.prototype.createModal = function (){
    
    var self = this;
    var optionsSet = self.options;
    var selectionType = this.selectionType;
    
    var triggerMatchTypeUrl;

    if(selectionType==="exec"){
        if(typeof optionsSet.devicetypefilter === "undefined" || optionsSet.devicetypefilter === null){
            triggerMatchTypeUrl = '{"jsonrpc": "2.0", "method": "TriggerService.getTriggerActionTypes", "params":{"filter":[]},"id":"TriggerService.getTriggerActionTypes"}';
        } else {
            triggerMatchTypeUrl = '{"jsonrpc": "2.0", "method": "TriggerService.getTriggerActionTypes", "params":{"filter":'+JSON.stringify(optionsSet.devicetypefilter)+'},"id":"TriggerService.getTriggerActionTypes"}';
        }
    } else if(selectionType==="match"){
        if(typeof optionsSet.devicetypefilter === "undefined" || optionsSet.devicetypefilter === null){
            triggerMatchTypeUrl = '{"jsonrpc": "2.0", "method": "TriggerService.getTriggerMatchTypes", "params":{"filter":[]},"id":"TriggerService.getTriggerMatchTypes"}';
        } else {
            triggerMatchTypeUrl = '{"jsonrpc": "2.0", "method": "TriggerService.getTriggerMatchTypes", "params":{"filter":'+JSON.stringify(optionsSet.devicetypefilter)+'},"id":"TriggerService.getTriggerMatchTypes"}';
        }
    } else {
        if(typeof optionsSet.devicetypefilter === "undefined" || optionsSet.devicetypefilter === null){
            triggerMatchTypeUrl = '{"jsonrpc": "2.0", "method": "TriggerService.getAllTriggerTypes", "params":{"filter":[]},"id":"TriggerService.getAllTriggerTypes"}';
        } else {
            triggerMatchTypeUrl = '{"jsonrpc": "2.0", "method": "TriggerService.getAllTriggerTypes", "params":{"filter":'+JSON.stringify(optionsSet.devicetypefilter)+'},"id":"TriggerService.getAllTriggerTypes"}';
        }
    }
    $("#ItemSelectionModalTypes").empty();
    getHttpJsonRPC(triggerMatchTypeUrl, 
        function(data){
            for(var i=0;i<data.length;i++){
                $("#ItemSelectionModalTypes").append('<a href="#'+data[i].id+'" class="list-group-item ItemSelectionModalTypesSelection">'+data[i].name+'</a>');
            }
            $(".ItemSelectionModalTypesSelection").on("click", function(){
                $("#ItemSelectionModalTypeName").text($(this).text());
                $("#itemSelectionMatchContent").empty();
                $('#addMatchItem').hide();
                switch ($(this).attr("href").replace("#", "")) {
                    case "remotesplugin":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="itemMatchRemotePlugin"><option value="">Choose remote</option></select></div>');
                        $('#itemMatchRemotePlugin').combobox();
                        getHttpJsonRPC('{"jsonrpc": "2.0", "method": "RemotesService.getRemotes","id":"RemotesService.getRemotes"}', function(pluginData){
                            for(var i=0; i<pluginData.length;i++){
                                $("#itemMatchRemotePlugin").append('<option value="'+pluginData[i].id+'">'+pluginData[i].name+'</option>');
                            }
                            $('#itemMatchRemotePlugin').combobox("refresh");
                            $("#itemMatchRemotePlugin").on('change', function(event) {
                                if ($('#itemMatchRemotePlugin').val()!=="") {
                                    $('#addMatchItem').show();
                                    self.callBackData = { "itemType": "remotesplugin", 
                                                          "remoteId": $('#itemMatchRemotePlugin').val(), 
                                                        "remoteName": $("#itemMatchRemotePlugin option:selected").text()
                                                    };
                                } else {
                                    $('#addMatchItem').hide();
                                    self.callBackData = {};
                                }
                            });
                        });
                    break;  
                    case "messengerplugin":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="itemMatchMessageType"><option value="">Choose message type</option></select></div>');
                        $('#itemMatchMessageType').combobox();
                        getHttpJsonRPC('{"jsonrpc": "2.0", "method": "MessengerService.getMessageTypes","id":"MessengerService.getMessageTypes"}', function(pluginData){
                            for(var i=0; i<pluginData.length;i++){
                                $("#itemMatchMessageType").append('<option value="'+pluginData[i].id+'">'+pluginData[i].name+'</option>');
                            }
                            $('#itemMatchMessageType').combobox("refresh");
                            $("#itemMatchMessageType").on('change', function(event) {
                                if ($('#itemMatchMessageType').val()!=="") {
                                    $('#addMatchItem').show();
                                    self.callBackData = { "itemType": "messengerplugin", 
                                                       "messageType": $('#itemMatchMessageType').val(), 
                                                          "typeName": $("#itemMatchMessageType option:selected").text(), 
                                                           "message":"" };
                                } else {
                                    $('#addMatchItem').hide();
                                    self.callBackData = {};
                                }
                            });
                        });
                    break;  
                    case "mediaplugin":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="itemMatchMediaPlugin"><option value="">Select a Media plugin</option></select></div>' +
                                '<div class="form-group"><label>Select command type</label><select class="form-control" id="itemMatchMediaCommandType"><option value="">Select media plugin first</option></select></div>');
                        $('#itemMatchMediaPlugin').combobox();
                        $('#itemMatchMediaCommandType').combobox();
                        $("#itemMatchMediaCommandType").combobox("disable");
                        getHttpJsonRPC('{"jsonrpc": "2.0", "method": "MediaService.getPlugins","id":"MediaService.getPlugins"}', function(pluginData){
                            for(var i=0; i<pluginData.length;i++){
                                $("#itemMatchMediaPlugin").append('<option value="'+pluginData[i].id+'">'+pluginData[i].name+'</option>');
                            }
                            $('#itemMatchMediaPlugin').combobox("refresh");
                            $("#itemMatchMediaPlugin").off("change").on('change', function(event) {
                                if($("#itemMatchMediaPlugin").val()===""){
                                    $("#itemMatchMediaCommandType").combobox("disable");
                                    $('#addMatchItem').hide();
                                } else {
                                    getHttpJsonRPC('{"jsonrpc": "2.0", "method": "MediaService.getCapabilities","params":{"id":'+$("#itemMatchMediaPlugin").val()+', "type":"commandset"}, "id":"MediaService.getCapabilities"}', function(pluginData){
                                        $("#itemMatchMediaCommandType").html('<option value="">Select command type</option>');
                                        for(var i=0; i<pluginData.length;i++){
                                            $("#itemMatchMediaCommandType").append('<option value="'+pluginData[i].id+'">'+pluginData[i].name+'</option>');
                                        }
                                        $('#itemMatchMediaCommandType').combobox("refresh");
                                        $("#itemMatchMediaCommandType").combobox("enable");
                                        $("#itemMatchMediaCommandType").off("change").on('change', function(event) {
                                            if($("#itemMatchMediaCommandType").val()===""){
                                                $('#addMatchItem').hide();
                                                self.callBackData = {};
                                            } else {
                                                $('#addMatchItem').show();
                                                self.callBackData = {
                                                    "itemType" : "mediaplugin",
                                                    "mediaId" : $("#itemMatchMediaPlugin").val(),
                                                    "mediaName" : $("#itemMatchMediaPlugin option:selected").text(),
                                                    "mediaCommandTypeId" : $("#itemMatchMediaCommandType").val(),
                                                    "mediaCommandTypeName" : $("#itemMatchMediaCommandType option:selected").text()
                                                };
                                            }
                                        });
                                    });
                                }
                            });
                        });
                    break;
                    case "device":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="itemMatchDevice"><option value="">Select a Device</option></select></div>' +
                                '<div class="form-group"><label>'+((self.selectionType==='exec')?'Select control to run':'Select control to match against')+'</label><select class="form-control" id="itemMatchDevicegroup"><option value="">Select device first</option></select></div>');
                        $('#itemMatchDevice').combobox();
                        $('#itemMatchDevicegroup').combobox();
                        $("#itemMatchDevicegroup").combobox("disable");
                        $('#itemMatchDevicegroupaction').combobox();
                        $("#itemMatchDevicegroupaction").combobox("disable");
                        getHttpJsonRPC('{"jsonrpc": "2.0", "method": "DeviceService.getDeclaredDevices","id":"DeviceService.getDeclaredDevices"}', function(deviceData){
                            for(var i=0; i<deviceData.length;i++){
                                $("#itemMatchDevice").append('<option value="'+deviceData[i].id+'">'+deviceData[i].locationname+' - '+deviceData[i].name+'</option>');
                            }
                            $('#itemMatchDevice').combobox("refresh");
                            $("#itemMatchDevice").on('change', function(event) {
                                var deviceId = $(this).val();
                                var deviceName = $("#itemMatchDevice option:selected").text();
                                if (deviceId!=="") {
                                    try {
                                        $("#itemMatchDevicegroup").off('change');
                                        $("#itemMatchDevicegroup").html('<option value="">Select group</option>');
                                        $('#itemMatchDevicegroup').combobox("refresh");
                                        $("#itemMatchDevicegroup").combobox("enable");
                                        $("#itemMatchDevicegroupaction").off('change');
                                        $("#itemMatchDevicegroupaction").html('<option value="">Select group first</option>');
                                        $("#itemMatchDevicegroupaction").combobox("refresh");
                                    } catch (err) {
                                        /// no combobox yet.
                                    }
                                    getHttpJsonRPC('{"jsonrpc": "2.0", "method": "DeviceService.getDeviceActionGroups", "params":{"id":' + deviceId + ', "filter":'+((typeof optionsSet.devicefilter==="undefined")?'[]':'["' + optionsSet.devicefilter + '"]')+'},"id":"DeviceService.getDeviceActionGroups"}', function(data){
                                        for(var i=0; i<data.length;i++){
                                            for(var j=0; j < data[i].controls.length;j++){
                                                var control = data[i].controls[j];
                                                $("#itemMatchDevicegroup").append('<option value="'+data[i].id+'&'+control.typedetails.id+'">'+data[i].name+': '+control.typedetails.label+'</option>');
                                            }
                                        }
                                        $('#itemMatchDevicegroup').combobox("refresh");
                                        $("#itemMatchDevicegroup").on('change', function(event) {
                                            var combinedId = $(this).val().split("&");
                                            var groupId = combinedId[0];
                                            var controlId = combinedId[1];
                                            for(var i=0; i<data.length;i++){
                                                if(data[i].id===groupId){
                                                    for(var j=0; j < data[i].controls.length;j++){
                                                        var control = data[i].controls[j];
                                                        if(control.typedetails.id===controlId){
                                                            var groupName = data[i].name;
                                                            self.callBackData = {
                                                                "itemType" : "device",
                                                                "deviceId" : deviceId,
                                                                "deviceGroupId" : groupId,
                                                                "deviceCommandId" : controlId,
                                                                "deviceName" : deviceName,
                                                                "deviceGroupName" : groupName,
                                                                "deviceCommandName" : control.typedetails.label,
                                                                "deviceCommandValue" : control.typedetails.value,
                                                                "deviceCommandPrefix": ((typeof control.typedetails.prefix==="undefined")?"":control.typedetails.prefix),
                                                                "deviceCommandSuffix" : ((typeof control.typedetails.suffix==="undefined")?"":control.typedetails.suffix),
                                                                "deviceCommandDataType" : ((typeof control.typedetails.datatype==="undefined")?"":control.typedetails.datatype),
                                                                "deviceCommandType" : control.commandtype,
                                                                "deviceCommandSet" : control.typedetails.commandset,
                                                                "deviceCommandMinValue": ((typeof control.typedetails.minvalue==="undefined")?"":control.typedetails.minvalue),
                                                                "deviceCommandMaxValue": ((typeof control.typedetails.maxvalue==="undefined")?"":control.typedetails.maxvalue),
                                                                "deviceCommandWarnValue": ((typeof control.typedetails.warnvalue==="undefined")?"":control.typedetails.warnvalue),
                                                                "deviceCommandHighValue": ((typeof control.typedetails.highvalue==="undefined")?"":control.typedetails.highvalue),
                                                            };
                                                            $('#addMatchItem').show();
                                                            break;
                                                        }
                                                    }
                                                    break;
                                                }
                                            }
                                        });
                                    });
                                } else {
                                    $('#addMatchItem').hide();
                                    $("#itemMatchDevicegroup").off('change');
                                    $("#itemMatchDevicegroup").html('<option value="">Select device first</option>');
                                    $('#itemMatchDevicegroup').combobox("refresh");
                                    $("#itemMatchDevicegroup").combobox("disable");
                                    $("#itemMatchDevicegroupaction").off('change');
                                    $("#itemMatchDevicegroupaction").html('<option value="">Select group first</option>');
                                    $("#itemMatchDevicegroupaction").combobox("refresh");
                                    $("#itemMatchDevicegroupaction").combobox("disable");
                                }
                            });
                        });
                    break;
                    case "daytime":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="triggermatchitemdaytimereccurrence">'+
                                                            '<option value="">Select day reccurrence</option>' +
                                                            '<option value="ALL">Every day</option>' +
                                                            '<option value="WEEKDAY">Weekdays</option>' +
                                                            '<option value="WEEKEND">Weekends</option>' +
                                                            '<option value="MON">Monday</option>' +
                                                            '<option value="TUE">Tuesday</option>' +
                                                            '<option value="WED">Wednesday</option>' +
                                                            '<option value="THU">Thursday</option>' +
                                                            '<option value="FRI">Friday</option>' +
                                                            '<option value="SAT">Saturday</option>' +
                                                            '<option value="SUN">Sunday</option>' +
                                                         '</select>'+
                                '</div>' +
                                '<div class="form-group"><select class="form-control" id="triggermatchitemdaytimedaytime">'+
                                                            '<option value="">Select when</option>' +
                                                            '<option value="FIXED">Fixed time</option>' +
                                                            '<option value="SUNRISE">At sunrise</option>' +
                                                            '<option value="SUNSET">At sunset</option>' +
                                                         '</select>'+
                                '</div>');
                        $("#triggermatchitemdaytimereccurrence").combobox();
                        $("#triggermatchitemdaytimedaytime").combobox();
                        $("#triggermatchitemdaytimedaytime").combobox("disable");
                        $('#addMatchItem').hide();
                        $("#triggermatchitemdaytimereccurrence").on('change', function(event) {
                            if($("#triggermatchitemdaytimereccurrence").val()===""){
                                $("#triggermatchitemdaytimedaytime").combobox("disable");
                                $('#addMatchItem').hide();
                            } else {
                                $("#triggermatchitemdaytimedaytime").combobox("enable");
                            }
                            $("#triggermatchitemdaytimedaytime").off('change').on('change', function(event) {
                                if($("#triggermatchitemdaytimedaytime").val()===""){
                                    self.callBackData = {};
                                    $('#addMatchItem').hide();
                                } else {
                                    $('#addMatchItem').show();
                                    self.callBackData = {
                                        "itemType" : "daytime",
                                        "reccurrenceId" : $("#triggermatchitemdaytimereccurrence").val(),
                                        "daytimeItemId" : $("#triggermatchitemdaytimedaytime").val(),
                                        "reccurrenceName" : $("#triggermatchitemdaytimereccurrence option:selected").text(),
                                        "daytimeItemName" : $("#triggermatchitemdaytimedaytime option:selected").text()
                                    };
                                }
                            });
                        });
                    break;
                    case "macros":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="triggermatchitemmacroselect">'+
                                                            '<option value="">Select macro to run</option>' +
                                                         '</select>'+
                                '</div>');
                        $("#triggermatchitemmacroselect").combobox();
                        getHttpJsonRPC('{"jsonrpc": "2.0", "method": "MacroService.getMacros","id":"MacroService.getMacros"}', function(macroData){
                            for(var i=0; i<macroData.length;i++){
                                $("#triggermatchitemmacroselect").append('<option value="'+macroData[i].id+'">'+macroData[i].name+'</option>');
                            }
                            $("#triggermatchitemmacroselect").combobox("refresh");
                            $("#triggermatchitemmacroselect").off("change").on('change', function(event) {
                                if($("#triggermatchitemmacroselect").val()!==""){
                                    $('#addMatchItem').show();
                                    self.callBackData = {
                                        "itemType" : "macro",
                                        "macroId" : $("#triggermatchitemmacroselect").val(),
                                        "macroName" : $("#triggermatchitemmacroselect option:selected").text()
                                    };
                                } else {
                                    $('#addMatchItem').hide();
                                    self.callBackData = {};
                                }
                            });
                        });
                    break;
                    case "scenes":
                        $("#itemSelectionMatchContent").html(
                                '<div class="form-group"><select class="form-control" id="triggermatchitemsceneselect">'+
                                                            '<option value="">Select scene to run</option>' +
                                                         '</select>'+
                                '</div>');
                        $("#triggermatchitemsceneselect").combobox();
                        getHttpJsonRPC('{"jsonrpc": "2.0", "method": "ScenesService.getScenes","id":"ScenesService.getScenes"}', function(scenesData){
                            for(var i=0; i<scenesData.length;i++){
                                $("#triggermatchitemsceneselect").append('<option value="'+scenesData[i].id+'">'+scenesData[i].name+'</option>');
                            }
                            $("#triggermatchitemsceneselect").combobox("refresh");
                            $("#triggermatchitemsceneselect").off("change").on('change', function(event) {
                                if($("#triggermatchitemsceneselect").val()!==""){
                                    $('#addMatchItem').show();
                                    self.callBackData = {
                                        "itemType" : "scene",
                                        "sceneId" : $("#triggermatchitemsceneselect").val(),
                                        "sceneName" : $("#triggermatchitemsceneselect option:selected").text()
                                    };
                                } else {
                                    $('#addMatchItem').hide();
                                    self.callBackData = {};
                                }
                            });
                        });
                    break;
                    case "presence":
                        $("#itemSelectionMatchContent").html('<p>Click add item to add presence options</p>');
                        $('#addMatchItem').show();
                        self.callBackData = { "itemType" : "presence", "matchType" : "", "matchValue": 0 };
                    break;
                    case "daypart":
                        $("#itemSelectionMatchContent").html('<p>Click add item to add day part options</p>');
                        $('#addMatchItem').show();
                        self.callBackData = { "itemType" : "daypart", "matchType" : "", "matchValue": 0 };
                    break;
                    case "userstatus":
                        $("#itemSelectionMatchContent").html('<p>Click add item to add user status options</p>');
                        $('#addMatchItem').show();
                        self.callBackData = { "itemType" : "userstatus", "matchType" : "", "matchValue": 0 };
                    break;
                    case "currenttime":
                        $("#itemSelectionMatchContent").html('<p>Click add item to add current time options</p>');
                        $('#addMatchItem').show();
                        self.callBackData = { "itemType" : "currenttime", "matchType" : "", "matchValue": 0 };
                    break;
                    case "weatherplugin":
                        $("#itemSelectionMatchContent").html('<p>Click add item to add weather options</p>');
                        $('#addMatchItem').show();
                        self.callBackData = { "itemType" : "weatherplugin", "matchType" : "", "matchValue": 0 };
                    break;
                    case "utilityusages":
                        $("#itemSelectionMatchContent").html('<p>Click add item to add utility usages options</p>');
                        $('#addMatchItem').show();
                        self.callBackData = { "itemType" : "utilityusages", "matchType" : "", "matchValue": 0 };
                    break;
                }
            });
        }  
    );
    $("#itemSelectionMatchContent").empty();
};

    function getWeatherIcon(weatherPluginData){
        var returnBegin = 'wi ';
        var returnIcon  = 'wi-day-sunny';
        switch(weatherPluginData.icon){
            case 'CLEAR':
                returnIcon  = 'wi-day-sunny';
            break;
            case 'MOSTLY_CLEAR':
                returnIcon  = 'wi-day-cloudy';
            break;
            case 'PARTLY_CLEAR':
                returnIcon  = 'wi-day-sunny-overcast';
            break;
            case 'INTERMITTENT_CLOUDS':
                returnIcon  = 'wi-day-cloudy-high';
            break;
            case 'HAZE':
                returnIcon  = 'wi-day-haze';
            break;
            case 'MOSTLY_CLOUDY':
                returnIcon  = 'wi-day-cloudy';
            break;
            case 'CLOUDY':
                returnIcon  = 'wi-day-cloudy';
            break;
            case 'DREARY':
                returnIcon  = 'wi-raindrops';
            break;
            case 'FOG':
                returnIcon  = 'wi-day-fog';
            break;
            case 'SHOWERS':
                returnIcon  = 'wi-day-showers';
            break;
            case 'MOSTLY_CLOUDY_SHOWERS':
                returnIcon  = 'wi-day-rain';
            break;
            case 'PARTLY_CLEAR_SHOWERS':
                returnIcon  = 'wi-day-showers';
            break;
            case 'THUNDERSTORMS':
                returnIcon  = 'wi-day-storm-showers';
            break;
            case 'MOSTLY_CLOUDY_THUNDERSTORMS':
                returnIcon  = 'wi-day-thunderstorm';
            break;
            case 'PARTLY_CLEAR_THUNDERSTORMS':
                returnIcon  = 'wi-day-lightning';
            break;
            case 'RAIN':
                returnIcon  = 'wi-day-rain';
            break;
            case 'FLURRIES':
                returnIcon  = 'wi-day-snow';
            break;
            case 'MOSTLY_CLOUDY_FLURRIES':
                returnIcon  = 'wi-day-snow';
            break;
            case 'PARTLY_CLEAR_FLURRIES':
                returnIcon  = 'wi-day-snow';
            break;
            case 'SNOW':
                returnIcon  = 'wi-snow';
            break;
            case 'MOSTLY_CLOUDY_SNOW':
                returnIcon  = 'wi-day-snow';
            break;
            case 'ICE':
                returnIcon  = 'wi-snowflake-cold';
            break;
            case 'SLEET':
                returnIcon  = 'wi-day-sleet';
            break;
            case 'FREEZING_RAIN':
                returnIcon  = 'wi-sleet';
            break;
            case 'RAIN_AND_SNOW':
                returnIcon  = 'wi-day-rain-mix';
            break;
            case 'EXTREME_HOT':
                returnIcon  = 'wi-hot';
            break;
            case 'EXTREME_COLD':
                returnIcon  = 'wi-snowflake-cold';
            break;
            case 'WINDY':
                returnIcon  = 'wi-strong-wind';
            break;
            /// night
            case 'CLEAR_NIGHT':
                returnIcon  = 'wi-night-clear';
            break;
            case 'MOSTLY_CLEAR_NIGHT':
                returnIcon  = 'wi-night-partly-cloudy';
            break;
            case 'PARTLY_CLEAR_NIGHT':
                returnIcon  = 'wi-night-partly-cloudy';
            break;
            case 'INTERMITTENT_CLOUDS_NIGHT':
                returnIcon  = 'wi-night-cloudy-high';
            break;
            case 'HAZE_NIGHT':
                returnIcon  = 'wi-fog';
            break;
            case 'MOSTLY_CLOUDY_NIGHT':
                returnIcon  = 'wi-night-cloudy';
            break;
            case 'PARTLY_CLEAR_SHOWERS_NIGHT':
                returnIcon  = 'wi-night-cloudy';
            break;
            case 'MOSTLY_CLOUDY_SHOWERS_NIGHT':
                returnIcon  = 'wi-night-showers';
            break;
            case 'MOSTLY_CLOUDY_THUNDERSTORMS_NIGHT':
                returnIcon  = 'wi-night-thunderstorm';
            break;
            case 'PARTLY_CLEAR_THUNDERSTORMS_NIGHT':
                returnIcon  = 'wi-night-lightning';
            break;
            case 'MOSTLY_CLOUDY_FLURRIES_NIGHT':
                returnIcon  = 'wi-night-snow';
            break;
            case 'MOSTLY_CLOUDY_SNOW_NIGHT':
                returnIcon  = 'wi-night-snow';
            break;
            case 'SMOG':
                returnIcon  = 'wi-smog';
            break;
            case 'SAND':
                returnIcon  = 'wi-dust';
            break;
            case 'TORNADO':
                returnIcon  = 'wi-tornado';
            break;
            case 'HAIL':
                returnIcon  = 'wi-hail';
            break;
        }
        return returnBegin + returnIcon;
    }