/**
 * Graphing class using HighCharts.
 * @param {type} divId
 * @param {type} serverLocation
 * @param {type} itemId
 * @param {type} groupId
 * @param {type} controlId
 * @param {type} description
 * @param {type} datacattype
 * @param {type} sourceType
 * @param {type} graphType
 * @param {type} decimals
 * @param {type} title
 * @param {type} clean
 * @returns {Graphing}
 */

function BasicGraph(divId, serverLocation, itemId, groupId, controlId, description, datacattype, sourceType, graphType, decimals, title, clean){
    this.divId          = divId;
    this.serverLocation = serverLocation;
    this.itemId         = itemId;
    this.groupId        = groupId;
    this.itemCategory   = controlId;
    this.description    = description;
    this.width          = 1024;
    this.height         = 512;
    this.sourceType     = sourceType;
    this.decimals       = decimals;
    this.dataUrl        = "";
    this.choiceIndex    = 1;
    if(clean===undefined){
        this.onlyGraph = false;
    } else if (clean===false){
        this.onlyGraph = false;
    } else {
        this.onlyGraph = true;
    }
    if(title===undefined){
        this.title = null;
    } else {
        this.title = title;
    }
    this.yAxisType = 'value';
    if(graphType==="time-log"){
        this.graphType = "line";
        this.yAxisType = "log";
    } else if (graphType==="time-totals"){
        this.graphType = "bar";
    } else if (graphType==="time-series"){
        this.graphType = "line";
    }
    
    if(graphType==="time-log"){
        this.decimals = 4;
    }
    
    switch(datacattype){
        case "TEMP":
            this.datacat = "Temperature in 'C";
        break;
        case "LUX":
            this.datacat = "Light intensity in Lux";
        break;
        case "MB":
            this.datacat = "Mega Bytes";
        break;
        case "PERC":
            this.datacat = "Percentage";
        break;
        case "TOTALS":
            this.datacat = "Totals";
        break;
        default:
            this.datacat = datacattype;
        break;
    }
    
    this.dataSelectSource = [
        "hour",
        "day",
        "week",
        "month",
        "year"
    ];
    
    this.descriptionSet = [
        "Showing last hour",
        "Showing last day",
        "Last week in weekdays",
        "Last month in days",
        "Last year in weeknumbers"
    ];

};

BasicGraph.prototype.getOptions = function(categoriesData, dataSet){
    var self = this;
    var data = {
        tooltip: {
            trigger: 'axis'
        },
        legend: {
            x: 220,
            y: 40,
            data: ['Something']
        },
        toolbox: {
            show: true,
            orient: 'horizontal',
            itemGap: 5,
            right: 1,
            iconStyle: {
                normal: {
                    textPosition: 'left'
                }
            },
            feature: {
                magicType: {
                    show: true,
                    title: {
                        line: 'Line',
                        bar: 'Bar'
                    },
                    type: ['line', 'bar']
                },
                saveAsImage: {
                    show: true,
                    title: "Save Image"
                }
            }
        },
        calculable: true,
        xAxis: [{
                type: 'category',
                data: categoriesData,
                axisLabel : {
                    formatter: function (value) {return self.dateFormat(value, self.dataSelectSource[self.choiceIndex]); }
                }
            }],
        yAxis: [{
                type: this.yAxisType,
                scale : true
            }],
        series: [{
                name: this.datacat,
                type: this.graphType,
                smooth: true,
                itemStyle: {
                    normal: {
                        areaStyle: {
                            type: 'default'
                        }
                    }
                },
                data: dataSet,
                mapValuePrecision: self.decimals
            }]
    };
    if(this.onlyGraph){
        data.title = {
            text: this.title,
            subtext: this.description
        }
    }
    return data;
};

BasicGraph.prototype.dateFormat = function(theDate, selection){
    switch(selection){
        case "hour":
        case "day":
            return theDate.toLocaleTimeString({ hour: '2-digit', minute: '2-digit', hour12: false});
        break;
        case "week":
            return theDate.toLocaleDateString({ weekday: 'short'});
        break;
        case "month":
            return theDate.toLocaleDateString({ year: '2-digit', month: '2-digit', day: '2-digit' });
        break;
        case "year":
            return theDate.toLocaleDateString({ year: '2-digit', month: '2-digit', day: '2-digit' });
        break;
    }
    return theDate.toLocaleString();
}

BasicGraph.prototype.createTooltip = function(){
    
}

BasicGraph.prototype.getTheme = function (){
    return {
          color: [
              '#26B99A', '#34495E', '#BDC3C7', '#3498DB',
              '#9B59B6', '#8abb6f', '#759c6a', '#bfd3b7'
          ],

          title: {
              itemGap: 8,
              textStyle: {
                  fontWeight: 'normal',
                  color: '#408829'
              }
          },

          dataRange: {
              color: ['#1f610a', '#97b58d']
          },

          toolbox: {
              color: ['#408829', '#408829', '#408829', '#408829']
          },

          tooltip: {
              backgroundColor: 'rgba(0,0,0,0.5)',
              axisPointer: {
                  type: 'line',
                  lineStyle: {
                      color: '#408829',
                      type: 'dashed'
                  },
                  crossStyle: {
                      color: '#408829'
                  },
                  shadowStyle: {
                      color: 'rgba(200,200,200,0.3)'
                  }
              }
          },

          dataZoom: {
              dataBackgroundColor: '#eee',
              fillerColor: 'rgba(64,136,41,0.2)',
              handleColor: '#408829'
          },
          grid: {
              borderWidth: 0
          },

          categoryAxis: {
              axisLine: {
                  lineStyle: {
                      color: '#408829'
                  }
              },
              splitLine: {
                  lineStyle: {
                      color: ['#eee']
                  }
              }
          },

          valueAxis: {
              axisLine: {
                  lineStyle: {
                      color: '#408829'
                  }
              },
              splitArea: {
                  show: true,
                  areaStyle: {
                      color: ['rgba(250,250,250,0.1)', 'rgba(200,200,200,0.1)']
                  }
              },
              splitLine: {
                  lineStyle: {
                      color: ['#eee']
                  }
              }
          },
          timeline: {
              lineStyle: {
                  color: '#408829'
              },
              controlStyle: {
                  normal: {color: '#408829'},
                  emphasis: {color: '#408829'}
              }
          },

          k: {
              itemStyle: {
                  normal: {
                      color: '#68a54a',
                      color0: '#a9cba2',
                      lineStyle: {
                          width: 1,
                          color: '#408829',
                          color0: '#86b379'
                      }
                  }
              }
          },
          map: {
              itemStyle: {
                  normal: {
                      areaStyle: {
                          color: '#ddd'
                      },
                      label: {
                          textStyle: {
                              color: '#c12e34'
                          }
                      }
                  },
                  emphasis: {
                      areaStyle: {
                          color: '#99d2dd'
                      },
                      label: {
                          textStyle: {
                              color: '#c12e34'
                          }
                      }
                  }
              }
          },
          force: {
              itemStyle: {
                  normal: {
                      linkStyle: {
                          strokeColor: '#408829'
                      }
                  }
              }
          },
          chord: {
              padding: 4,
              itemStyle: {
                  normal: {
                      lineStyle: {
                          width: 1,
                          color: 'rgba(128, 128, 128, 0.5)'
                      },
                      chordStyle: {
                          lineStyle: {
                              width: 1,
                              color: 'rgba(128, 128, 128, 0.5)'
                          }
                      }
                  },
                  emphasis: {
                      lineStyle: {
                          width: 1,
                          color: 'rgba(128, 128, 128, 0.5)'
                      },
                      chordStyle: {
                          lineStyle: {
                              width: 1,
                              color: 'rgba(128, 128, 128, 0.5)'
                          }
                      }
                  }
              }
          },
          gauge: {
              startAngle: 225,
              endAngle: -45,
              axisLine: {
                  show: true,
                  lineStyle: {
                      color: [[0.2, '#86b379'], [0.8, '#68a54a'], [1, '#408829']],
                      width: 8
                  }
              },
              axisTick: {
                  splitNumber: 10,
                  length: 12,
                  lineStyle: {
                      color: 'auto'
                  }
              },
              axisLabel: {
                  textStyle: {
                      color: 'auto'
                  }
              },
              splitLine: {
                  length: 18,
                  lineStyle: {
                      color: 'auto'
                  }
              },
              pointer: {
                  length: '90%',
                  color: 'auto'
              },
              title: {
                  textStyle: {
                      color: '#333'
                  }
              },
              detail: {
                  textStyle: {
                      color: 'auto'
                  }
              }
          },
          textStyle: {
              fontFamily: 'Arial, Verdana, sans-serif'
          }
      };

}



BasicGraph.prototype.updateConfig = function ( choiceIndex ){
    
    var self = this;
    
    var urlToUse = "getDeviceGraph";
    
    this.choiceIndex = choiceIndex;
    
    switch(this.sourceType){
        case "DEVICE":
            urlToUse = "getDeviceGraph";
        break;
        case "PLUGIN":
            urlToUse = "getUtilityGraph";
        break;
    }
    
    this.dataUrl = '/jsonrpc.json?rpc={"jsonrpc": "2.0", "method": "GraphService.'+urlToUse+'", "params":{"id":'+this.itemId+',"group":"'+this.groupId+'","control":"' + this.itemCategory + '","range":["'+this.dataSelectSource[choiceIndex]+'"], "calculation":null}, "id":"GraphService.'+urlToUse+'"}';
    
    $.get(this.dataUrl, function(resultData) {
        var seriesData = [];
        var categoriesData = [];
        for(var i=0; i < resultData.result.data[self.dataSelectSource[self.choiceIndex]].length;i++){
            var dataSet = resultData.result.data[self.dataSelectSource[self.choiceIndex]][i];
            categoriesData.push(
                new Date(dataSet.key)
            );
            seriesData.push(
                dataSet.value
            );
        }
        
        self.chart = echarts.init(document.getElementById(self.divId + "_chart"), self.getTheme());
        self.chart.setOption(self.getOptions(categoriesData, seriesData));
        
        $(window).on('resize', function(){
            if(self.chart !== null && self.chart !== undefined){
                self.chart.resize();
            }
        });
        
    });
    
};

BasicGraph.prototype.updateGraph = function (datacollectionIndex){
    this.updateConfig(datacollectionIndex);
};

BasicGraph.prototype.createGraph = function (width, height, startOptionIndex){
    this.width = width;
    this.height= height;
    if(startOptionIndex===undefined){
        this.updateConfig(1);
    } else {
        this.updateConfig(startOptionIndex);
    }
    
    var uid = createUUID();
    
    $('#' + this.divId).empty();
    if(this.onlyGraph){
        $('#' + this.divId).html('<div class="devicechart" id="'+this.divId+'_chart"></div>');
    } else {
        $('#' + this.divId).html('<div class="row"><div class="col-xs-12"><div class="x_panel tile">\
                    <div class="x_title">\
                        <h2>'+this.title+' <small>'+this.description+'</small></h2>\
                        <a href="'+encodeURI(this.dataUrl)+'" download="'+this.title+'.json" data-toggle="tooltip" data-placement="top" title="Download data" class="close-link pull-right blocked-pointer-container hidden-sm hidden-xs hidden-md" style="margin-left: 5px;">\
                            <i class="fa fa-download"></i>\
                        </a>\
                        <div class="pull-right">\
                            <div id="reportrange" class="dropdown blocked-pointer-container">\
                                <i class="glyphicon glyphicon-calendar fa fa-calendar"></i>&nbsp;\
                                <a id="drop4" href="#" class="dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" role="button" aria-expanded="false">\
                                    <span id="graphDataSourceName_'+uid+'"></span>&nbsp;\
                                    <b><span class="caret"></span></b>\
                                </a>\
                                <ul id="graphDataSource_'+uid+'" class="dropdown-menu">\
                                    <li role="presentation" data-value="0"><a role="menuitem" tabindex="-1" href="javascript:;">Last hour</a></li>\
                                    <li role="presentation" data-value="1"><a role="menuitem" tabindex="-1" href="javascript:;">Last 24 hours</a></li>\
                                    <li role="presentation" data-value="2"><a role="menuitem" tabindex="-1" href="javascript:;">Last week</a></li>\
                                    <li role="presentation" data-value="3"><a role="menuitem" tabindex="-1" href="javascript:;">Last month</a></li>\
                                    <li role="presentation" data-value="4"><a role="menuitem" tabindex="-1" href="javascript:;">Last year</a></li>\
                                </ul>\
                            </div>\
                        </div>\
                        <div class="clearfix"></div>\
                    </div>\
                    <div class="x_content">\
                        <div class="row">\
                            <div class="col-xs-12">\
                                <div class="devicechart" id="'+this.divId+'_chart" style="width:100%; height:'+$('#' + this.divId).width()/2+'px;"></div>\
                            </div>\
                        </div>\
                    </div>\
                </div></div></div>');        
    }
    
    var self = this;
    
    if(!this.onlyGraph){
        $('#graphDataSourceName_'+uid).text($('#graphDataSource_'+uid).find("[data-value='"+this.choiceIndex+"']").find('a').text());
        $('#graphDataSource_'+uid+' li').off('click').on('click', function(event) {
            $('#graphDataSourceName_'+uid).text($(this).find('a').text());
            self.updateGraph(parseInt($(this).attr('data-value')));
        });
        $("#graphdataselect_"+uid).off('click').on("click", function(){
            window.open(self.dataUrl,'_blank');
        });
    }
    
    if(this.sourceType=="DEVICE"){
        pidomeRPCSocket.addCallback(function(params) {
            try {
                var paramSet = params.params;
                if(self.itemId === paramSet.id){
                    for(var i=0;i<paramSet.groups.length;i++){
                        if(self.groupId === paramSet.groups[i].groupid){
                            for(var controlSetId in paramSet.groups[i].controls){
                                if(controlSetId === self.itemCategory){
                                    if (self.dataSelectSource[self.choiceIndex] === "hour" && self.chart!==undefined) {
                                        var data = self.chart.getOption().series[0].data;
                                        var xAxis = self.chart.getOption().xAxis[0].data;
                                        
                                        xAxis.shift();
                                        xAxis.push(new Date());
                                        
                                        data.shift();
                                        data.push(paramSet.groups[i].controls[controlSetId]);
                                        self.chart.setOption({
                                                series: { data : data },
                                                xAxis: { data: xAxis }
                                            });
                                    }
                                    break;
                                }
                            }
                            break;
                        }
                    }
                }
            } catch (err) {
                console.log(err);
            }
        }, "DeviceService.sendDevice");
    }
    
};