/**
 * Get an http rpc call.
 * @param {type} rpcCall
 * @param {type} callback
 * @returns {undefined}
 */
function getHttpJsonRPC(rpcCall, callback, silent) {
    $.get('/jsonrpc.json?rpc=' + rpcCall).done(function (data) {
        handleJsonResultData(data, rpcCall, callback, silent);
    });
}

function handleJsonResultData(response, rpcCall, callback, silent) {
    var beQuiet = (silent !== undefined && silent === true);
    if (typeof response.error === 'undefined') {
        if (response.result.success !== true) {
            if (!beQuiet)
                extendedPageError("Error executing request", response.result.data.message, null, rpcCall, response.result.data.trace);
        } else {
            if (typeof callback !== 'undefined' && callback !== null) {
                callback(response.result.data);
            }
        }
    } else {
        if (!beQuiet){
            if (typeof response.error.data.trace !== 'undefined'){
                extendedPageError("Error executing request", response.error.message + "(" + response.error.code + ")", response.error.data.message, rpcCall, response.error.data.trace);
            } else {
                extendedPageError("Error executing request", response.error.message + "(" + response.error.code + ")", response.error.data.message, rpcCall);
            }
        }
    }
}

function postHttpJsonRPC(rpcCall, callback, silent) {
    $.post('/jsonrpc.json', {rpc: rpcCall}).done(function (data) {
        handleJsonResultData(data, rpcCall, callback, silent);
    });
}

function returnResultData(response, rpcCall) {
    if (typeof response.error === 'undefined') {
        if (response.result.success !== true) {
            extendedPageError("Error executing request", response.result.data.message, null, (typeof rpcCall!=='undefined')?rpcCall:null, response.result.data.trace);
        } else {
            return response.result.data;
        }
    } else {
        if (typeof response.error.data.trace !== 'undefined'){
            extendedPageError("Error executing request", response.error.message + "(" + response.error.code + ")", response.error.data.message, (typeof rpcCall!=='undefined')?rpcCall:null, response.error.data.trace);
        } else {
            extendedPageError("Error executing request", response.error.message + "(" + response.error.code + ")", response.error.data.message, (typeof rpcCall!=='undefined')?rpcCall:null);
        }
    }
    return [];
}